﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace WellTracks
{
    public partial class Form1 : Form
    {
        private LayerOverlay overlay;
        private ShapeFileFeatureLayer wellTopLayer;
        private ShapeFileFeatureLayer wellBtmLayer;
        private InMemoryFeatureLayer wellTrackLayer;
        private Dictionary<string, LineShape> surveys;

        public Form1()
        {
            InitializeComponent();

            wellTopLayer = new ShapeFileFeatureLayer("../../app_data/wells_top.shp");
            wellBtmLayer = new ShapeFileFeatureLayer("../../app_data/wells_btm.shp");

            wellTopLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimpleCircleStyle(GeoColor.StandardColors.White, 20, GeoColor.StandardColors.Black, 2);
            wellTopLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            wellBtmLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimpleCircleStyle(GeoColor.StandardColors.White, 12, GeoColor.StandardColors.Orange, 1);
            wellBtmLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            wellTrackLayer = new InMemoryFeatureLayer();
            wellTrackLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.SimpleColors.Green, 2, false);
            wellTrackLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            overlay = new LayerOverlay();
            overlay.Layers.Add(wellTrackLayer);
            overlay.Layers.Add(wellBtmLayer);
            overlay.Layers.Add(wellTopLayer);
            winformsMap1.Overlays.Add(overlay);

            wellTopLayer.Open();
            wellBtmLayer.Open();
            winformsMap1.CurrentExtent = ExtentHelper.GetBoundingBoxOfItems(new Layer[] { wellTopLayer, wellBtmLayer }.Select(l => l.GetBoundingBox()));
            winformsMap1.Refresh();
        }

        private void button1_Click(object sender, System.EventArgs e)
        {
            InitalizeSurveys();
            wellTrackLayer.InternalFeatures.Clear();

            string[] returnColumnNames = new string[] { "RECID" };
            Collection<Feature> topWells = wellTopLayer.QueryTools.GetAllFeatures(returnColumnNames);
            foreach (var topWell in topWells)
            {
                string wellId = topWell.ColumnValues["RECID"];
                Feature bottomWell = wellBtmLayer.QueryTools.GetFeatureById(wellId, returnColumnNames);
                if (bottomWell != null && IsSlantWell(topWell, bottomWell))
                {
                    LineShape slantTrack = new LineShape();
                    PointShape topSpot = (PointShape)topWell.GetShape();
                    PointShape bottomSpot = (PointShape)bottomWell.GetShape();
                    slantTrack.Vertices.Add(new Vertex(topSpot.X, topSpot.Y));
                    slantTrack.Vertices.Add(new Vertex(bottomSpot.X, bottomSpot.Y));

                    if (surveys.ContainsKey(wellId))
                    {
                        foreach (var v in surveys[wellId].Vertices.Reverse())
                        {
                            slantTrack.Vertices.Insert(1, new Vertex(v.X, v.Y));
                        }
                    }

                    Feature feature = new Feature(slantTrack.GetWellKnownBinary(), topWell.Id);
                    wellTrackLayer.InternalFeatures.Add(feature);
                }
            }

            winformsMap1.Refresh(overlay);
        }

        private void InitalizeSurveys()
        {
            if (surveys == null)
            {
                surveys = new Dictionary<string, LineShape>();
                foreach (var line in File.ReadAllLines("../../app_data/Survey.txt").Skip(1))
                {
                    try
                    {
                        var segments = line.Split(new char[] { '\t' }, StringSplitOptions.RemoveEmptyEntries);
                        if (segments.Length == 2)
                        {
                            surveys[segments[0].Trim()] = new LineShape(segments[1].Trim());
                        }
                    }
                    catch { }
                }
            }
        }

        private static bool IsSlantWell(Feature top, Feature bottom)
        {
            bool isSlant = false;
            if (!top.GetWellKnownText().Equals(bottom.GetWellKnownText(), StringComparison.OrdinalIgnoreCase))
            {
                isSlant = true;
            }

            return isSlant;
        }
    }
}